import { Version } from '@angular/core';
import { JsonObject } from '@angular-devkit/core';
import { Rule, Tree } from '@angular-devkit/schematics';
import { ApplicationDefinition, LibraryDefinition } from './chainable/chainable-project';
/**
 * Ensures that the workspace, where the schematic is currently running on, is actually an
 * Angular workspace or throws an exception otherwise. The test is done by ensuring the
 * existence of an `angular.json` file.
 * @throws {SchematicsException} An exception if an `angular.json` file was not found.
 * @returns {Rule}
 */
export declare const ensureIsAngularWorkspace: () => Rule;
/**
 * Ensures that a project is actually an Angular application or throws an exception otherwise.
 * @throws {SchematicsException} An exception if the project is not an Angular application.
 * @param {string} projectName The name of the project to look for.
 * @returns {Rule}
 */
export declare const ensureIsAngularApplication: (projectName: string) => Rule;
/**
 * Ensures that a project is actually an Angular library or throws an exception otherwise.
 * @throws {SchematicsException} An exception if the project is not an Angular library.
 * @param {string} projectName The name of the project to look for.
 * @returns {Rule}
 */
export declare const ensureIsAngularLibrary: (projectName: string) => Rule;
/**
 * Executes a rule only if the current Angular version installed in the project satisfies a given range.
 * @param {string} range An Angular version range that must be satisfied.
 * @param {Rule} rule The rule to execute.
 * @returns {Rule}
 */
export declare const isAngularVersion: (range: string, rule: Rule) => Rule;
/**
 * Adds a new asset to the `build` and `test` sections of the `angular.json` file.
 * @param {JsonObject|string} value The asset to add.
 * @param {string} projectName The name of the project to look for.
 * @returns {Rule}
 */
export declare const addAngularJsonAsset: (value: JsonObject | string, projectName: string) => Rule;
/**
 * Removes an asset from the `build` and `test` sections of the `angular.json` file.
 * @param {JsonObject|string} value The asset to remove.
 * @param {string} projectName The name of the project to look for.
 * @returns {Rule}
 */
export declare const removeAngularJsonAsset: (value: JsonObject | string, projectName: string) => Rule;
/**
 * Adds a new style to the `build` and `test` sections of the `angular.json` file.
 * @param {JsonObject|string} value The style to add.
 * @param {string} projectName The name of the project to look for.
 * @returns {Rule}
 */
export declare const addAngularJsonStyle: (value: JsonObject | string, projectName: string) => Rule;
/**
 * Removes a style from the `build` and `test` sections of the `angular.json` file.
 * @param {JsonObject|string} value The style to remove.
 * @param {string} projectName The name of the project to look for.
 * @returns {Rule}
 */
export declare const removeAngularJsonStyle: (value: JsonObject | string, projectName: string) => Rule;
/**
 * Adds a new script to the `build` and `test` sections of the `angular.json` file.
 * @param {JsonObject|string} value The script to add.
 * @param {string} projectName The name of the project to look for.
 * @returns {Rule}
 */
export declare const addAngularJsonScript: (value: JsonObject | string, projectName: string) => Rule;
/**
* Removes a script from the `build` and `test` sections of the `angular.json` file.
* @param {JsonObject|string} value The style to remove.
* @param {string} projectName The name of the project to look for.
* @returns {Rule}
*/
export declare const removeAngularJsonScript: (value: JsonObject | string, projectName: string) => Rule;
/**
 * Inserts a declaration (ex. Component, Pipe, Directive) into an NgModule declarations and also imports that declaration.
 * @param {string} filePath The path of the file containing the NgModule to modify.
 * @param {string} classifiedName The classified name of the declaration to insert.
 * @param {string} importPath The path of the file containing the declaration to insert.
 * @returns {Rule}
 */
export declare const addDeclarationToNgModule: (filePath: string, classifiedName: string, importPath: string) => Rule;
/**
 * Removes a declaration (ex. Component, Pipe, Directive) from an NgModule declarations.
 * @param {string} filePath The path of the file containing the NgModule to modify.
 * @param {string} classifiedName The classified name of the declaration to remove.
 * @returns {Rule}
 */
export declare const removeDeclarationFromNgModule: (filePath: string, classifiedName: string) => Rule;
/**
 * Inserts an import (ex. NgModule) into an NgModule imports and also imports that import.
 * @param {string} filePath The path of the file containing the NgModule to modify.
 * @param {string} classifiedName The classified name of the import to insert.
 * @param {string} importPath The path of the file containing the import to insert.
 * @returns {Rule}
 */
export declare const addImportToNgModule: (filePath: string, classifiedName: string, importPath: string) => Rule;
/**
 * Removes an import (ex. NgModule) from an NgModule imports.
 * @param {string} filePath The path of the file containing the NgModule to modify.
 * @param {string} classifiedName The classified name of the import to remove.
 * @returns {Rule}
 */
export declare const removeImportFromNgModule: (filePath: string, classifiedName: string) => Rule;
/**
 * Inserts an export (ex. Component, Pipe, Directive) into an NgModule exports and also imports that export.
 * @param {string} filePath The path of the file containing the NgModule to modify.
 * @param {string} classifiedName The classified name of the export to insert.
 * @param {string} importPath The path of the file containing the export to insert.
 * @returns {Rule}
 */
export declare const addExportToNgModule: (filePath: string, classifiedName: string, importPath: string) => Rule;
/**
 * Removes an export (ex. Component, Pipe, Directive) from an NgModule exports.
 * @param {string} filePath The path of the file containing the NgModule to modify.
 * @param {string} classifiedName The classified name of the export to remove.
 * @returns {Rule}
 */
export declare const removeExportFromNgModule: (filePath: string, classifiedName: string) => Rule;
/**
 * Inserts a provider (ex. Service) into an NgModule providers and also imports that provider.
 * @param {string} filePath The path of the file containing the NgModule to modify.
 * @param {string} classifiedName The classified name of the provider to insert.
 * @param {string} importPath The path of the file containing the provider to insert.
 * @returns {Rule}
 */
export declare const addProviderToNgModule: (filePath: string, classifiedName: string, importPath: string) => Rule;
/**
 * Removes a provider (ex. Service) from an NgModule providers.
 * @param {string} filePath The path of the file containing the NgModule to modify.
 * @param {string} classifiedName The classified name of the provider to remove.
 * @returns {Rule}
 */
export declare const removeProviderFromNgModule: (filePath: string, classifiedName: string) => Rule;
/**
 * Inserts a route declaration to a router module (i.e. as a RouterModule declaration).
 * @param {string} filePath The path of the file containing the NgModule to modify.
 * @param {string} routeLiteral The Route object to insert.
 * @returns {Rule}
 */
export declare const addRouteDeclarationToNgModule: (filePath: string, routeLiteral: string) => Rule;
/**
 * Inserts a provider (ex. provideRouter) into a bootstrapApplication's providers and also imports that provider.
 * @param {string} filePath The path of the file containing the bootstrapApplication call to modify.
 * @param {string} providerName The name of the provider to insert.
 * @param {string} importPath The path of the file containing the provider to insert.
 * @param {boolean} [useImportProvidersFrom=false] Whether or not to use the importProvidersFrom api.
 * @param {number} [indent=2] The indentation used during the insertion.
 * @returns {Rule}
 */
export declare const addProviderToBootstrapApplication: (filePath: string, providerName: string, importPath: string, useImportProvidersFrom?: boolean, indent?: number) => Rule;
/**
 * Removes a provider (ex. provideRouter) from a bootstrapApplication's providers.
 * @param {string} filePath The path of the file containing the bootstrapApplication call to modify.
 * @param {string} providerName The name of the provider to remove.
 * @returns {Rule}
 */
export declare const removeProviderFromBootstrapApplication: (filePath: string, providerName: string) => Rule;
/**
 * Gets the version of Angular currently used in the project.
 * @returns {Version}
 */
export declare const getAngularVersion: () => Version;
/**
 * Gets a project output path as defined in the `angular.json` file.
 * @param {Tree} tree The current schematic's project tree.
 * @param {string} projectName The name of the project to look for.
 * @throws {SchematicsException} An exception if a `defaultProject` property was not found in the `angular.json` file.
 * @returns {string} The default project output path.
 */
export declare const getProjectOutputPath: (tree: Tree, projectName: string) => string;
/**
 * Gets a project main file path as defined in the `angular.json` file.
 * @param {Tree} tree The current schematic's project tree.
 * @param {string} projectName The name of the project to look for.
 * @returns {string} The default project main file path.
 */
export declare const getProjectMainFilePath: (tree: Tree, projectName: string) => string;
/**
 * Gets a standalone project main config file path.
 * @param {Tree} tree The current schematic's project tree.
 * @param {string} projectName The name of the project to look for.
 * @returns {string|null} The project main config file path if found or null otherwise.
 */
export declare const getProjectMainConfigFilePath: (tree: Tree, projectName: string) => string | null;
/**
 * Gets a project definition object from the current Angular workspace.
 * @async
 * @param {Tree} tree The current schematic's project tree.
 * @param {string} projectName The name of the project to look for.
 * @throws {SchematicsException} An exception if no project was found.
 * @returns {Promise<LibraryDefinition | ApplicationDefinition>} A project definition object.
 */
export declare const getProjectFromWorkspace: <T extends LibraryDefinition | ApplicationDefinition>(tree: Tree, projectName: string) => Promise<T>;
/**
 * Checks if a project if of type standalone.
 * @param {Tree} tree The current schematic's project tree.
 * @param {string} projectName The name of the project to look for.
 * @returns {boolean} Whether or not the project is of type standalone.
 */
export declare const isProjectStandalone: (tree: Tree, projectName: string) => boolean;
/**
 * @internal
 */
export declare const ensureProjectIsDefined: (projectName: string | undefined) => void;
