"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getStandaloneApplicationConfig = exports.addProviderToStandaloneApplication = exports.removeProviderFromStandaloneApplication = exports.removeSymbolFromNgModuleMetadata = exports.getIndentBy = exports.guessLineIndentationAtPosition = void 0;
const schematics_1 = require("@angular-devkit/schematics");
const typescript_1 = require("@schematics/angular/third_party/github.com/Microsoft/TypeScript/lib/typescript");
const ast_utils_1 = require("@schematics/angular/utility/ast-utils");
const change_1 = require("@schematics/angular/utility/change");
const app_config_1 = require("@schematics/angular/utility/standalone/app_config");
const util_1 = require("@schematics/angular/utility/standalone/util");
const file_1 = require("./file");
/**
 * @internal
 */
const guessLineIndentationAtPosition = (sourceFile, position) => {
    const { line, character } = sourceFile.getLineAndCharacterOfPosition(position);
    const lineStartPosition = sourceFile.getPositionOfLineAndCharacter(line, 0);
    const lineStartToPosition = sourceFile.getText().substring(lineStartPosition, lineStartPosition + character);
    const lineIndentation = lineStartToPosition.search(/\S/);
    return (lineIndentation >= 0) ? lineIndentation : 0;
};
exports.guessLineIndentationAtPosition = guessLineIndentationAtPosition;
/**
 * @internal
 */
const getIndentBy = (tree, filePath, position) => {
    const sourceFile = (0, file_1.getTsSourceFile)(tree, filePath);
    const lineIndent = (0, exports.guessLineIndentationAtPosition)(sourceFile, position);
    return (n) => ' '.repeat(lineIndent + n);
};
exports.getIndentBy = getIndentBy;
/**
 * @internal
 */
const removeSymbolFromNgModuleMetadata = (sourceFile, filePath, metadataField, classifiedName) => {
    const ngModuleNodes = (0, ast_utils_1.getDecoratorMetadata)(sourceFile, 'NgModule', '@angular/core');
    const ngModuleImports = (0, ast_utils_1.getMetadataField)(ngModuleNodes[0], metadataField);
    const arrayLiteral = ngModuleImports[0].initializer;
    const symbolIndex = arrayLiteral.elements.findIndex(el => el.getText().includes(classifiedName));
    if (symbolIndex !== -1) {
        const el = arrayLiteral.elements[symbolIndex];
        let position = el.getFullStart();
        let fullText = el.getFullText();
        if (symbolIndex !== (arrayLiteral.elements.length - 1)) {
            fullText = `${fullText},`;
        }
        else if (arrayLiteral.elements.length > 1) {
            position--;
            fullText = `,${fullText}`;
        }
        return new change_1.RemoveChange(filePath, position, fullText);
    }
    return new change_1.NoopChange();
};
exports.removeSymbolFromNgModuleMetadata = removeSymbolFromNgModuleMetadata;
/**
 * @internal
 */
const removeProviderFromStandaloneApplication = (tree, mainFilePath, providerName) => {
    var _a;
    const bootstrapApplicationCall = (0, util_1.findBootstrapApplicationCall)(tree, mainFilePath);
    if (bootstrapApplicationCall) {
        const boostrapApplicationOptions = (_a = bootstrapApplicationCall.arguments) === null || _a === void 0 ? void 0 : _a[1];
        // Options is a variable => new appConfig object introduces with ng16
        if (boostrapApplicationOptions && (0, typescript_1.isIdentifier)(boostrapApplicationOptions)) {
            const appConfig = (0, app_config_1.findAppConfig)(bootstrapApplicationCall, tree, mainFilePath);
            if (appConfig) {
                (0, file_1.commitChanges)(tree, appConfig.filePath, removeProviderFromConfig(appConfig.filePath, appConfig.node, providerName));
            }
            else {
                throw new schematics_1.SchematicsException('Could not find application config');
            }
            // Options is an object
        }
        else if (boostrapApplicationOptions && (0, typescript_1.isObjectLiteralExpression)(boostrapApplicationOptions)) {
            (0, file_1.commitChanges)(tree, mainFilePath, removeProviderFromConfig(mainFilePath, boostrapApplicationOptions, providerName));
        }
    }
    else {
        throw new schematics_1.SchematicsException(`Could not find bootstrapApplication call in ${mainFilePath}`);
    }
};
exports.removeProviderFromStandaloneApplication = removeProviderFromStandaloneApplication;
/**
 * @internal
 */
const addProviderToStandaloneApplication = (tree, mainFilePath, providerName, useImportProvidersFrom = false, indent = 2) => {
    var _a, _b;
    const bootstrapApplicationCall = (0, util_1.findBootstrapApplicationCall)(tree, mainFilePath);
    if (bootstrapApplicationCall) {
        const boostrapApplicationOptions = (_a = bootstrapApplicationCall.arguments) === null || _a === void 0 ? void 0 : _a[1];
        // Options is a variable => new appConfig object introduced with ng16
        if (boostrapApplicationOptions && (0, typescript_1.isIdentifier)(boostrapApplicationOptions)) {
            const appConfig = (0, app_config_1.findAppConfig)(bootstrapApplicationCall, tree, mainFilePath);
            if (appConfig) {
                (0, file_1.commitChanges)(tree, appConfig.filePath, [addProviderToConfig(tree, appConfig.filePath, appConfig.node, providerName, useImportProvidersFrom, indent)]);
            }
            else {
                throw new schematics_1.SchematicsException('Could not find application config');
            }
            // Options is an object
        }
        else if (boostrapApplicationOptions && (0, typescript_1.isObjectLiteralExpression)(boostrapApplicationOptions)) {
            (0, file_1.commitChanges)(tree, mainFilePath, [addProviderToConfig(tree, mainFilePath, boostrapApplicationOptions, providerName, useImportProvidersFrom, indent)]);
            // No options
        }
        else {
            const indentBy = (0, exports.getIndentBy)(tree, mainFilePath, bootstrapApplicationCall.getStart());
            const indentedProviderName = providerName.replace(/\r?\n|\r/gm, `$&${indentBy(indent * 2)}`);
            const toAdd = `, {\n${indentBy(indent)}providers: [\n${indentBy(indent * 2)}${indentedProviderName}\n${indentBy(indent)}]\n${indentBy(0)}}`;
            (0, file_1.commitChanges)(tree, mainFilePath, [new change_1.InsertChange(mainFilePath, (_b = bootstrapApplicationCall.arguments) === null || _b === void 0 ? void 0 : _b[0].getEnd(), toAdd)]);
        }
    }
    else {
        throw new schematics_1.SchematicsException(`Could not find bootstrapApplication call in ${mainFilePath}`);
    }
};
exports.addProviderToStandaloneApplication = addProviderToStandaloneApplication;
/**
 * @internal
 */
const getStandaloneApplicationConfig = (tree, mainFilePath) => {
    var _a;
    try {
        const bootstrapApplicationCall = (0, util_1.findBootstrapApplicationCall)(tree, mainFilePath);
        if (bootstrapApplicationCall) {
            const boostrapApplicationOptions = (_a = bootstrapApplicationCall.arguments) === null || _a === void 0 ? void 0 : _a[1];
            if (boostrapApplicationOptions && (0, typescript_1.isIdentifier)(boostrapApplicationOptions)) {
                return (0, app_config_1.findAppConfig)(bootstrapApplicationCall, tree, mainFilePath);
            }
        }
        return null;
    }
    catch {
        return null;
    }
};
exports.getStandaloneApplicationConfig = getStandaloneApplicationConfig;
// --- HELPER(s) ---
/**
 * @internal
 */
const removeProviderFromConfig = (filePath, config, providerName) => {
    var _a;
    const providersArrayProp = (_a = config.properties) === null || _a === void 0 ? void 0 : _a.find(prop => ((0, typescript_1.isPropertyAssignment)(prop) &&
        (0, typescript_1.isIdentifier)(prop.name) &&
        (0, typescript_1.isArrayLiteralExpression)(prop.initializer) &&
        (prop.name.text === 'providers')));
    if (providersArrayProp) {
        const providersArray = providersArrayProp.initializer;
        return providersArray.elements
            .map((prop, index) => {
            if (prop.getText().includes(providerName)) {
                if ((0, typescript_1.isCallExpression)(prop) && (0, typescript_1.isIdentifier)(prop.expression) && (prop.expression.text === 'importProvidersFrom')) {
                    if (prop.arguments.length > 1) {
                        const newProviders = prop.arguments.map(arg => arg.getText()).filter(name => (name !== providerName));
                        if (newProviders.length) {
                            const toReplace = `importProvidersFrom(${newProviders.join(', ')})`;
                            return new change_1.ReplaceChange(filePath, prop.getStart(), prop.getText(), toReplace);
                        }
                    }
                }
                const toRemove = (index === (providersArray.elements.length - 1)) ? prop.getFullText() : `${prop.getFullText()},`;
                return new change_1.RemoveChange(filePath, prop.getFullStart(), toRemove);
            }
            return new change_1.NoopChange();
        });
    }
    return [];
};
/**
 * @internal
 */
const addProviderToConfig = (tree, filePath, config, providerName, useImportProvidersFrom = false, indent = 2) => {
    var _a, _b;
    const indentBy = (0, exports.getIndentBy)(tree, filePath, config.getStart());
    const providersArrayProp = (_a = config.properties) === null || _a === void 0 ? void 0 : _a.find(prop => ((0, typescript_1.isPropertyAssignment)(prop) &&
        (0, typescript_1.isIdentifier)(prop.name) &&
        (0, typescript_1.isArrayLiteralExpression)(prop.initializer) &&
        (prop.name.text === 'providers')));
    if (providersArrayProp) {
        const providersArray = providersArrayProp.initializer;
        if (useImportProvidersFrom) {
            const importProvidersFromProp = providersArray.elements.find(prop => (0, typescript_1.isCallExpression)(prop) &&
                (0, typescript_1.isIdentifier)(prop.expression) &&
                (prop.expression.text === 'importProvidersFrom'));
            if (!importProvidersFromProp) {
                const toAdd = `\n${indentBy(indent * 2)}importProvidersFrom(${providerName})`;
                return new change_1.InsertChange(filePath, providersArray.getStart() + 1, (providersArray.elements.length) ? `${toAdd},` : toAdd);
            }
            else if (!importProvidersFromProp.getText().includes(providerName)) {
                return new change_1.InsertChange(filePath, (_b = importProvidersFromProp.arguments) === null || _b === void 0 ? void 0 : _b[0].getStart(), `${providerName}, `);
            }
            else {
                return new change_1.NoopChange();
            }
        }
        else {
            const indentedProviderName = providerName.replace(/\r?\n|\r/gm, `$&${indentBy(indent * 2)}`);
            const toAdd = `\n${indentBy(indent * 2)}${indentedProviderName}`;
            return new change_1.InsertChange(filePath, providersArray.getStart() + 1, (providersArray.elements.length) ? `${toAdd},` : toAdd);
        }
    }
    else {
        const indentedProviderName = providerName.replace(/\r?\n|\r/gm, `$&${indentBy(indent * 2)}`);
        const toAdd = `\n${indentBy(indent)}providers: [\n${indentBy(indent * 2)}${indentedProviderName}\n${indentBy(indent)}]`;
        return new change_1.InsertChange(filePath, config.getStart() + 1, (config.properties.length) ? `${toAdd},` : `${toAdd}\n`);
    }
};
