"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Chainable = exports.ChainableType = void 0;
const schematics_1 = require("@angular-devkit/schematics");
const workspace_1 = require("@schematics/angular/utility/workspace");
const angular_1 = require("../angular");
const file_1 = require("../file");
const rules_1 = require("../rules");
var ChainableType;
(function (ChainableType) {
    ChainableType[ChainableType["WORKSPACE"] = 0] = "WORKSPACE";
    ChainableType[ChainableType["APPLICATION"] = 1] = "APPLICATION";
    ChainableType[ChainableType["LIBRARY"] = 2] = "LIBRARY";
})(ChainableType || (exports.ChainableType = ChainableType = {}));
class Chainable {
    constructor(chainableType) {
        this.chainableType = chainableType;
        /*
         * We need an array of lazy loaded rules because we have to make sure that a rule has everything she needs.
         * (ex: access to project, tree, workspace, etc.)
         */
        this._rules = [];
        this.init();
    }
    /**
     * @see {@link deployFiles}
     * @returns {this}
     */
    deployFiles(templateOptions = {}, source = './files', destination = '', strategy = schematics_1.MergeStrategy.Overwrite) {
        return this.addRuleToChain(() => (0, file_1.deployFiles)(templateOptions, source, this.pathFromRoot(destination), strategy));
    }
    /**
     * @see {@link deleteFiles}
     * @returns {this}
     */
    deleteFiles(files, force = false) {
        return this.addRuleToChain(() => (0, file_1.deleteFiles)(files.map(file => this.pathFromRoot(file)), force));
    }
    /**
     * @see {@link renameFile}
     * @returns {this}
     */
    renameFile(from, to) {
        return this.addRuleToChain(() => (0, file_1.renameFile)(this.pathFromRoot(from), this.pathFromRoot(to)));
    }
    /**
     * @see {@link createOrUpdateFile}
     * @returns {this}
     */
    createOrUpdateFile(filePath, data) {
        return this.addRuleToChain(() => (0, file_1.createOrUpdateFile)(this.pathFromRoot(filePath), data));
    }
    /**
     * @see {@link downloadFile}
     * @returns {this}
     */
    downloadFile(source, destination, replace = false) {
        return this.addRuleToChain(() => (0, file_1.downloadFile)(source, this.pathFromRoot(destination), replace));
    }
    /**
     * @see {@link replaceInFile}
     * @returns {this}
     */
    replaceInFile(filePath, searchValue, replaceValue) {
        return this.addRuleToChain(() => (0, file_1.replaceInFile)(this.pathFromRoot(filePath), searchValue, replaceValue));
    }
    /**
     * @see {@link addImportToFile}
     * @returns {this}
     */
    addImportToFile(filePath, symbolName, fileName, isDefault) {
        return this.addRuleToChain(() => (0, file_1.addImportToFile)(this.pathFromRoot(filePath), symbolName, fileName, isDefault));
    }
    /**
     * @see {@link modifyImportInFile}
     * @returns {this}
     */
    modifyImportInFile(filePath, symbolName, newSymbolName, fileName) {
        return this.addRuleToChain(() => (0, file_1.modifyImportInFile)(this.pathFromRoot(filePath), symbolName, newSymbolName, fileName));
    }
    /**
     * @see {@link removeImportFromFile}
     * @returns {this}
     */
    removeImportFromFile(filePath, symbolName, fileName) {
        return this.addRuleToChain(() => (0, file_1.removeImportFromFile)(this.pathFromRoot(filePath), symbolName, fileName));
    }
    /**
     * @see {@link modifyJsonFile}
     * @returns {this}
     */
    modifyJsonFile(filePath, jsonPath, value, insertInOrder) {
        return this.addRuleToChain(() => (0, file_1.modifyJsonFile)(this.pathFromRoot(filePath), jsonPath, value, insertInOrder));
    }
    /**
     * @see {@link removeFromJsonFile}
     * @returns {this}
     */
    removeFromJsonFile(filePath, jsonPath) {
        return this.addRuleToChain(() => (0, file_1.removeFromJsonFile)(this.pathFromRoot(filePath), jsonPath));
    }
    /**
     * @see {@link isAngularVersion}
     * @returns {this}
     */
    isAngularVersion(range, rule) {
        return this.addRuleToChain(() => (0, angular_1.isAngularVersion)(range, () => rule(this.getContext())));
    }
    /**
     * @see {@link log}
     * @returns {this}
     */
    log(message) {
        return this.addRuleToChain(() => (0, rules_1.log)(message));
    }
    /**
     * @see {@link logInfo}
     * @returns {this}
     */
    logInfo(message) {
        return this.addRuleToChain(() => (0, rules_1.logInfo)(message));
    }
    /**
     * @see {@link logWarning}
     * @returns {this}
     */
    logWarning(message) {
        return this.addRuleToChain(() => (0, rules_1.logWarning)(message));
    }
    /**
     * @see {@link logError}
     * @returns {this}
     */
    logError(message) {
        return this.addRuleToChain(() => (0, rules_1.logError)(message));
    }
    /**
     * @see {@link logAction}
     * @returns {this}
     */
    logAction(message) {
        return this.addRuleToChain(() => (0, rules_1.logAction)(message));
    }
    /**
     * @see {@link spawn}
     * @returns {this}
     */
    spawn(command, args, showOutput = false) {
        return this.addRuleToChain(() => (0, rules_1.spawn)(command, args, showOutput));
    }
    /**
     * @see {@link runAtEnd}
     * @returns {this}
     */
    runAtEnd(rule) {
        return this.addRuleToChain(() => (0, rules_1.runAtEnd)(rule));
    }
    /**
     * Runs a custom rule within the chainable context.
     * @param {ChainableRule<T>} rule The custom rule to be run.
     * @returns {this}
     */
    rule(rule) {
        return this.addRuleToChain(() => () => rule(this.getContext()));
    }
    /**
     * Transforms the chainable context into an array of rules.
     * @returns {Rule[]}
     */
    toRules() {
        return this._rules;
    }
    /**
     * Transforms the chainable context into a single rule.
     * @returns {Rule}
     */
    toRule() {
        return (0, schematics_1.chain)(this._rules);
    }
    // --- HELPER(s) ---
    addRuleToChain(rule) {
        this._rules.push(rule);
        return this;
    }
    getContext() {
        return {
            tree: this._tree,
            schematicContext: this._schematicContext,
            workspace: this._workspace
        };
    }
    pathFromRoot(path) {
        return path;
    }
    init() {
        // Sanity checks
        this.addRuleToChain(angular_1.ensureIsAngularWorkspace);
        // Make sure the first thing we do in the chain is to get all the needed references
        this.addRuleToChain(() => async (tree, schematicContext) => {
            this._tree = tree;
            this._schematicContext = schematicContext;
            this._workspace = await (0, workspace_1.getWorkspace)(tree);
        });
    }
}
exports.Chainable = Chainable;
