import { JsonValue } from '@angular-devkit/core';
import { MergeStrategy, Rule, Tree } from '@angular-devkit/schematics';
import { SourceFile } from '@schematics/angular/third_party/github.com/Microsoft/TypeScript/lib/typescript';
import { Change } from '@schematics/angular/utility/change';
import { InsertionIndex, JSONPath } from '@schematics/angular/utility/json-file';
/**
 * Deploys assets files and optionally applies computation to them.
 * @param {{}} [templateOptions={}] A set of options to be applied on each `.template` file to deploy.
 * @param {string} [source="./files"] The path to the folder containing the files to deploy.
 * @param {string} [destination=""] The path to the destination folder of the deploy operation.
 * @param {MergeStrategy} [strategy=MergeStrategy.Overwrite] The merge strategy to apply on the files to deploy.
 * @returns {Rule}
 */
export declare const deployFiles: (templateOptions?: {}, source?: string, destination?: string, strategy?: MergeStrategy) => Rule;
/**
 * Deletes a collection of files or folders
 * @param {string[]} paths A set of file or folder paths to delete.
 * @param {boolean} [force=false] Force deletion (required for folders).
 * @returns {Rule}
 */
export declare const deleteFiles: (paths: string[], force?: boolean) => Rule;
/**
 * Rename a file
 * @param {string} from The path of the file to rename.
 * @param {string} to The new path of the file to rename.
 * @returns {Rule}
 */
export declare const renameFile: (from: string, to: string) => Rule;
/**
 * Creates or updates a file.
 * @param {string} filePath The path of the file to create or modify.
 * @param {unknown} data The data to be written to the file.
 * @returns {Rule}
 */
export declare const createOrUpdateFile: (filePath: string, data: unknown) => Rule;
/**
 * Downloads a file.
 * @param {string|URL} source The path or url to the filename to download.
 * @param {string} destination The path to the destination filename of the download operation.
 * @param {boolean} [replace=false] Whether or not to overwrite the destination file if it already exists.
 * @param {number} [retries=3] The number of times to retry the request in case of failure.
 * @param {number} [backoff=300] The delay (in milliseconds) between retries.
 * @returns {Rule}
 */
export declare const downloadFile: (source: string | URL, destination: string, replace?: boolean, retries?: number, backoff?: number) => Rule;
/**
 * Replaces text in a file, using a regular expression or a search string.
 * @param {string} filePath The path of the file to modify.
 * @param {string|RegExp} searchValue A string or RegExp to search for.
 * @param {string} replaceValue A string containing the text to replace for every successful match of searchValue.
 * @returns {Rule}
 */
export declare const replaceInFile: (filePath: string, searchValue: string | RegExp, replaceValue: string) => Rule;
/**
 * Adds an import to a file.
 * @param {string} filePath The path of the file to modify.
 * @param {string} symbolName The name of the item to import.
 * @param {string} fileName The path of the file containing the item to import.
 * @param {boolean} [isDefault] If true, the import will follow the style for importing default exports.
 * @returns {Rule}
 */
export declare const addImportToFile: (filePath: string, symbolName: string, fileName: string, isDefault?: boolean) => Rule;
/**
 * Modifies or removes an import inside a file.
 * @param {string} filePath The path of the file to modify.
 * @param {string} symbolName The name of the item to update or remove.
 * @param {string|undefined} newSymbolName The new name of the item or `undefined` if it is to be removed.
 * @param {string} fileName The path of the file containing the item to update or remove.
 * @returns {Rule}
 */
export declare const modifyImportInFile: (filePath: string, symbolName: string, newSymbolName: string | undefined, fileName: string) => Rule;
/**
 * Removes an import inside a file.
 * @param {string} filePath The path of the file to modify.
 * @param {string} symbolName The name of the item to remove.
 * @param {string} fileName The path of the file containing the item to remove.
 * @returns {Rule}
 */
export declare const removeImportFromFile: (filePath: string, symbolName: string, fileName: string) => Rule;
/**
 * Adds or modifies an element in a JSON file.
 * @param {string} filePath The path of the file to modify.
 * @param {JSONPath} jsonPath A path to the element to modify in the JSON structure (separated by dot notation).
 * @param {JsonValue|undefined} value The new value to assign to the element.
 * @param {InsertionIndex|false} [insertInOrder] Manage the insertion order.
 * @returns {Rule}
 */
export declare const modifyJsonFile: (filePath: string, jsonPath: JSONPath, value: JsonValue | undefined, insertInOrder?: InsertionIndex | false) => Rule;
/**
* Removes an element inside a JSON file.
* @param {string} filePath The path of the file to modify.
* @param {JSONPath} jsonPath A path to the element to remove in the JSON structure (separated by dot notation).
* @returns {Rule}
*/
export declare const removeFromJsonFile: (filePath: string, jsonPath: JSONPath) => Rule;
/**
 * Converts a JavaScript value to a JavaScript Object Notation (JSON) string.
 * Uses a default indentation of 2.
 * @param {unknown} value A JavaScript value, usually an object or array, to be converted.
 * @returns {string}
 */
export declare const serializeToJson: (value: unknown) => string;
/**
 * Gets the source of a TypeScript file.
 * @param {Tree} tree The current schematic's project tree.
 * @param {string} filePath The path of the file to get the source from.
 * @returns {Rule}
 */
export declare const getTsSourceFile: (tree: Tree, filePath: string) => SourceFile;
/**
 * Applies changes on a file inside the current schematic's project tree.
 * @param {Tree} tree The current schematic's project tree.
 * @param {string} filePath The path of the file to apply the changes to.
 * @param {Change[]} changes The changes to apply to the file.
 * @returns {void}
 */
export declare const commitChanges: (tree: Tree, filePath: string, changes: Change[]) => void;
