"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.commitChanges = exports.getTsSourceFile = exports.serializeToJson = exports.removeFromJsonFile = exports.modifyJsonFile = exports.removeImportFromFile = exports.modifyImportInFile = exports.addImportToFile = exports.replaceInFile = exports.downloadFile = exports.createOrUpdateFile = exports.renameFile = exports.deleteFiles = exports.deployFiles = void 0;
const core_1 = require("@angular-devkit/core");
const schematics_1 = require("@angular-devkit/schematics");
const typescript_1 = require("@schematics/angular/third_party/github.com/Microsoft/TypeScript/lib/typescript");
const ast_utils_1 = require("@schematics/angular/utility/ast-utils");
const change_1 = require("@schematics/angular/utility/change");
const json_file_1 = require("@schematics/angular/utility/json-file");
const request_1 = require("./request");
// --- RULES(s) ----
/**
 * Deploys assets files and optionally applies computation to them.
 * @param {{}} [templateOptions={}] A set of options to be applied on each `.template` file to deploy.
 * @param {string} [source="./files"] The path to the folder containing the files to deploy.
 * @param {string} [destination=""] The path to the destination folder of the deploy operation.
 * @param {MergeStrategy} [strategy=MergeStrategy.Overwrite] The merge strategy to apply on the files to deploy.
 * @returns {Rule}
 */
const deployFiles = (templateOptions = {}, source = './files', destination = '', strategy = schematics_1.MergeStrategy.Overwrite) => (0, schematics_1.mergeWith)((0, schematics_1.apply)((0, schematics_1.url)(source), [
    (0, schematics_1.applyTemplates)({
        utils: core_1.strings,
        ...templateOptions
    }),
    (0, schematics_1.move)((0, core_1.normalize)(destination))
]), strategy);
exports.deployFiles = deployFiles;
/**
 * Deletes a collection of files or folders
 * @param {string[]} paths A set of file or folder paths to delete.
 * @param {boolean} [force=false] Force deletion (required for folders).
 * @returns {Rule}
 */
const deleteFiles = (paths, force = false) => (tree) => {
    paths.forEach(path => {
        if (force) {
            try {
                tree.delete(path);
            }
            catch { /* swallow errors*/ }
        }
        else if (tree.exists(path)) {
            tree.delete(path);
        }
    });
};
exports.deleteFiles = deleteFiles;
/**
 * Rename a file
 * @param {string} from The path of the file to rename.
 * @param {string} to The new path of the file to rename.
 * @returns {Rule}
 */
const renameFile = (from, to) => (tree) => {
    if (tree.exists(from)) {
        tree.rename(from, to);
    }
};
exports.renameFile = renameFile;
/**
 * Creates or updates a file.
 * @param {string} filePath The path of the file to create or modify.
 * @param {unknown} data The data to be written to the file.
 * @returns {Rule}
 */
const createOrUpdateFile = (filePath, data) => (tree) => {
    var _a;
    if (!tree.exists(filePath)) {
        tree.create(filePath, (typeof data === 'string') ? data : (0, exports.serializeToJson)(data));
    }
    else {
        const actualData = (_a = tree.read(filePath)) === null || _a === void 0 ? void 0 : _a.toString('utf-8');
        const newData = (typeof data === 'string') ? data : (0, exports.serializeToJson)(data);
        if (actualData !== newData) {
            tree.overwrite(filePath, newData);
        }
    }
};
exports.createOrUpdateFile = createOrUpdateFile;
/**
 * Downloads a file.
 * @param {string|URL} source The path or url to the filename to download.
 * @param {string} destination The path to the destination filename of the download operation.
 * @param {boolean} [replace=false] Whether or not to overwrite the destination file if it already exists.
 * @param {number} [retries=3] The number of times to retry the request in case of failure.
 * @param {number} [backoff=300] The delay (in milliseconds) between retries.
 * @returns {Rule}
 */
const downloadFile = (source, destination, replace = false, retries = 3, backoff = 300) => async (tree) => {
    if (!tree.exists(destination) || replace) {
        const data = await (0, request_1.getDataFromUrl)(source, retries, backoff);
        if (!tree.exists(destination)) {
            tree.create(destination, data);
        }
        else {
            tree.overwrite(destination, data);
        }
    }
};
exports.downloadFile = downloadFile;
/**
 * Replaces text in a file, using a regular expression or a search string.
 * @param {string} filePath The path of the file to modify.
 * @param {string|RegExp} searchValue A string or RegExp to search for.
 * @param {string} replaceValue A string containing the text to replace for every successful match of searchValue.
 * @returns {Rule}
 */
const replaceInFile = (filePath, searchValue, replaceValue) => (tree) => {
    var _a, _b;
    if (tree.exists(filePath)) {
        const content = (_b = (_a = tree.read(filePath)) === null || _a === void 0 ? void 0 : _a.toString('utf-8')) !== null && _b !== void 0 ? _b : '';
        const newContent = content.replace(searchValue, replaceValue);
        if (content !== newContent) {
            tree.overwrite(filePath, newContent);
        }
    }
};
exports.replaceInFile = replaceInFile;
/**
 * Adds an import to a file.
 * @param {string} filePath The path of the file to modify.
 * @param {string} symbolName The name of the item to import.
 * @param {string} fileName The path of the file containing the item to import.
 * @param {boolean} [isDefault] If true, the import will follow the style for importing default exports.
 * @returns {Rule}
 */
const addImportToFile = (filePath, symbolName, fileName, isDefault) => (tree) => {
    const sourceFile = (0, exports.getTsSourceFile)(tree, filePath);
    const changes = (0, ast_utils_1.insertImport)(sourceFile, '', symbolName, fileName, isDefault);
    (0, exports.commitChanges)(tree, filePath, [changes]);
};
exports.addImportToFile = addImportToFile;
/**
 * Modifies or removes an import inside a file.
 * @param {string} filePath The path of the file to modify.
 * @param {string} symbolName The name of the item to update or remove.
 * @param {string|undefined} newSymbolName The new name of the item or `undefined` if it is to be removed.
 * @param {string} fileName The path of the file containing the item to update or remove.
 * @returns {Rule}
 */
const modifyImportInFile = (filePath, symbolName, newSymbolName, fileName) => (tree) => {
    const changes = [];
    const sourceFile = (0, exports.getTsSourceFile)(tree, filePath);
    const allImports = (0, ast_utils_1.findNodes)(sourceFile, typescript_1.SyntaxKind.ImportDeclaration);
    const relevantImports = allImports.filter(node => {
        const importFiles = node
            .getChildren()
            .filter(typescript_1.isStringLiteral)
            .map(n => n.text);
        return importFiles.filter(file => file === fileName).length === 1;
    });
    relevantImports.forEach(relevantImport => {
        const names = (0, ast_utils_1.findNodes)(relevantImport, typescript_1.SyntaxKind.Identifier);
        const symbolNameNodeIndex = names.findIndex(name => name.getText() === symbolName);
        if (symbolNameNodeIndex !== -1) {
            const symbolNameNode = names[symbolNameNodeIndex];
            if (newSymbolName) {
                // Rename the name in the import statement
                changes.push(new change_1.ReplaceChange(filePath, symbolNameNode.getStart(), symbolName, newSymbolName));
            }
            else if (names.length === 1) {
                // Remove the whole import statement
                changes.push(new change_1.RemoveChange(filePath, relevantImport.getFullStart(), relevantImport.getFullText()));
            }
            else {
                // Remove only the name from the import statement
                const newNames = names.filter(name => name.getText() !== symbolName).map(name => name.getText());
                const newRelevantImport = `import { ${newNames.join(', ')} } from '${fileName}';`;
                changes.push(new change_1.ReplaceChange(filePath, relevantImport.getStart(), relevantImport.getText(), newRelevantImport));
            }
        }
    });
    (0, exports.commitChanges)(tree, filePath, changes);
};
exports.modifyImportInFile = modifyImportInFile;
/**
 * Removes an import inside a file.
 * @param {string} filePath The path of the file to modify.
 * @param {string} symbolName The name of the item to remove.
 * @param {string} fileName The path of the file containing the item to remove.
 * @returns {Rule}
 */
const removeImportFromFile = (filePath, symbolName, fileName) => (0, exports.modifyImportInFile)(filePath, symbolName, undefined, fileName);
exports.removeImportFromFile = removeImportFromFile;
/**
 * Adds or modifies an element in a JSON file.
 * @param {string} filePath The path of the file to modify.
 * @param {JSONPath} jsonPath A path to the element to modify in the JSON structure (separated by dot notation).
 * @param {JsonValue|undefined} value The new value to assign to the element.
 * @param {InsertionIndex|false} [insertInOrder] Manage the insertion order.
 * @returns {Rule}
 */
const modifyJsonFile = (filePath, jsonPath, value, insertInOrder) => (tree) => {
    if (tree.exists(filePath)) {
        const jsonFile = new json_file_1.JSONFile(tree, filePath);
        if ((jsonPath.length === 0) && (JSON.stringify(JSON.parse(jsonFile.content)) === JSON.stringify(value))) {
            return;
        }
        else if (JSON.stringify(jsonFile.get(jsonPath)) === JSON.stringify(value)) {
            return;
        }
        else {
            jsonFile.modify(jsonPath, value, insertInOrder);
        }
    }
};
exports.modifyJsonFile = modifyJsonFile;
/**
* Removes an element inside a JSON file.
* @param {string} filePath The path of the file to modify.
* @param {JSONPath} jsonPath A path to the element to remove in the JSON structure (separated by dot notation).
* @returns {Rule}
*/
const removeFromJsonFile = (filePath, jsonPath) => (tree) => {
    if (tree.exists(filePath)) {
        const jsonFile = new json_file_1.JSONFile(tree, filePath);
        if (jsonFile.get(jsonPath) !== undefined) {
            jsonFile.remove(jsonPath);
        }
    }
};
exports.removeFromJsonFile = removeFromJsonFile;
// --- HELPER(s) ----
/**
 * Converts a JavaScript value to a JavaScript Object Notation (JSON) string.
 * Uses a default indentation of 2.
 * @param {unknown} value A JavaScript value, usually an object or array, to be converted.
 * @returns {string}
 */
const serializeToJson = (value) => `${JSON.stringify(value, null, 2)}\n`;
exports.serializeToJson = serializeToJson;
/**
 * Gets the source of a TypeScript file.
 * @param {Tree} tree The current schematic's project tree.
 * @param {string} filePath The path of the file to get the source from.
 * @returns {Rule}
 */
const getTsSourceFile = (tree, filePath) => {
    const buffer = tree.read(filePath);
    if (!buffer) {
        throw new schematics_1.SchematicsException(`File ${filePath} does not exist.`);
    }
    return (0, typescript_1.createSourceFile)(filePath, buffer.toString('utf-8'), typescript_1.ScriptTarget.Latest, true);
};
exports.getTsSourceFile = getTsSourceFile;
/**
 * Applies changes on a file inside the current schematic's project tree.
 * @param {Tree} tree The current schematic's project tree.
 * @param {string} filePath The path of the file to apply the changes to.
 * @param {Change[]} changes The changes to apply to the file.
 * @returns {void}
 */
const commitChanges = (tree, filePath, changes) => {
    const recorder = tree.beginUpdate(filePath);
    (0, change_1.applyToUpdateRecorder)(recorder, changes);
    tree.commitUpdate(recorder);
};
exports.commitChanges = commitChanges;
