"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getJsonFromUrl = exports.getDataFromUrl = void 0;
const https_1 = require("https");
/**
 * Returns the response data of a given url as a buffer object.
 * @async
 * @param {string|URL} url The url where to get the data from.
 * @param {number} [retries=3] The number of times to retry the request in case of failure.
 * @param {number} [backoff=300] The delay (in milliseconds) between retries.
 * @returns {Promise<Buffer>} A `Buffer` object.
 */
const getDataFromUrl = async (url, retries = 3, backoff = 300) => new Promise((resolve, reject) => {
    const { hostname, pathname } = (typeof url === 'string') ? new URL(url) : url;
    // eslint-disable-next-line consistent-return
    const req = (0, https_1.get)({ hostname, path: pathname }, (res) => {
        if (res.statusCode === 200) {
            const rawData = [];
            res.on('data', (chunk) => rawData.push(chunk));
            res.once('end', () => {
                res.removeAllListeners();
                try {
                    resolve(Buffer.concat(rawData));
                }
                catch (err) {
                    reject((err instanceof Error) ? err : new Error(String(err)));
                }
            });
        }
        else if (retries > 0) {
            setTimeout(() => void (0, exports.getDataFromUrl)(url, retries - 1, backoff * 2), backoff);
        }
        else {
            res.removeAllListeners();
            res.resume(); // consume response data to free up memory
            reject(new Error(`Request error (${String(res.statusCode)}): https://${hostname}${pathname}`));
        }
    });
    const abort = (error) => {
        if (retries > 0) {
            setTimeout(() => void (0, exports.getDataFromUrl)(url, retries - 1, backoff * 2), backoff);
        }
        else {
            req.removeAllListeners();
            req.destroy();
            reject((error instanceof Error) ? error : new Error(String(error)));
        }
    };
    req.once('timeout', () => abort(`Request timed out: https://${hostname}/${pathname}`));
    req.once('error', err => abort(err));
});
exports.getDataFromUrl = getDataFromUrl;
/**
 * Returns the response data of a given url as a JSON object.
 * @async
 * @param {string|URL} url The url where to get the data from.
 * @param {number} [retries=3] The number of times to retry the request in case of failure.
 * @param {number} [backoff=300] The delay (in milliseconds) between retries.
 * @returns {Promise<JsonObject>} A `JsonObject` object.
 */
const getJsonFromUrl = async (url, retries = 3, backoff = 300) => {
    const data = await (0, exports.getDataFromUrl)(url, retries, backoff);
    return JSON.parse(data.toString('utf-8'));
};
exports.getJsonFromUrl = getJsonFromUrl;
