"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSchematicSchemaDefaultOptions = exports.getSchematicSchemaOptions = void 0;
const json_schema_1 = require("@angular/cli/src/command-builder/utilities/json-schema");
const index_1 = require("@angular-devkit/core/src/json/schema/index");
const path_1 = require("path");
const request_1 = require("./request");
const getExternalSchemaJson = async (packageName, schematicName = 'ng-add', retries = 3, backoff = 300) => {
    var _a;
    const url = `http://cdn.jsdelivr.net/npm/${packageName}@latest`;
    const pkgJson = await (0, request_1.getJsonFromUrl)(path_1.posix.join(url, 'package.json'), retries, backoff);
    if (pkgJson === null || pkgJson === void 0 ? void 0 : pkgJson['schematics']) {
        const collectionJson = await (0, request_1.getJsonFromUrl)(path_1.posix.join(url, pkgJson['schematics']), retries, backoff);
        if (collectionJson === null || collectionJson === void 0 ? void 0 : collectionJson['schematics']) {
            const schema = (_a = collectionJson['schematics'][schematicName]) === null || _a === void 0 ? void 0 : _a['schema'];
            if (!schema) {
                throw new Error(`Schematic "${schematicName}" not found in collection "${packageName}".`);
            }
            return await (0, request_1.getJsonFromUrl)(path_1.posix.join(url, (0, path_1.dirname)(pkgJson['schematics']), schema), retries, backoff);
        }
    }
    throw new Error(`Package "${packageName}" was found but does not support schematics.`);
};
/**
 * Returns all the options of a specific local or external schematic's schema.
 * @async
 * @param {SchematicContext} context The current schematic context.
 * @param {string} [schematicName="ng-add"] Name of the schematic schema (ex: "drag-drop").
 * @param {string} [packageName="current-schematic-collection-name"] Name of the package containing the schematic schema (ex: "@angular/cli").
 * @param {boolean} [external=false] Whether the schematic is local or external.
 * @param {number} [retries=3] The number of times to retry the request in case of failure.
 * @param {number} [backoff=300] The delay (in milliseconds) between retries.
 * @returns {Promise<NgCliOption[]>} A collection of `NgCliOption` objects.
 */
const getSchematicSchemaOptions = async (context, schematicName = 'ng-add', packageName, external = false, retries = 3, backoff = 300) => {
    var _a, _b, _c, _d, _e;
    let schemaJson;
    if (!packageName) {
        packageName = (_c = (_b = (_a = context.schematic) === null || _a === void 0 ? void 0 : _a.collection) === null || _b === void 0 ? void 0 : _b.description) === null || _c === void 0 ? void 0 : _c.name;
    }
    if (!external) {
        const collection = context.engine.createCollection(packageName);
        const schematic = collection.createSchematic(schematicName, false);
        schemaJson = schematic.description.schemaJson;
    }
    else {
        schemaJson = await getExternalSchemaJson(packageName, schematicName, retries, backoff);
    }
    if (schemaJson === null || schemaJson === void 0 ? void 0 : schemaJson['properties']) {
        const schemaPropertiesOrdered = Object.keys(schemaJson === null || schemaJson === void 0 ? void 0 : schemaJson['properties']);
        const registry = (_e = (_d = context.engine.workflow) === null || _d === void 0 ? void 0 : _d.registry) !== null && _e !== void 0 ? _e : new index_1.CoreSchemaRegistry();
        const options = await (0, json_schema_1.parseJsonSchemaToOptions)(registry, schemaJson);
        options
            /**
             * Fix: @angular/cli is not keeping the options in the same order as they are declared in schema.json
             */
            .sort((a, b) => schemaPropertiesOrdered.indexOf(a.name) - schemaPropertiesOrdered.indexOf(b.name))
            /**
             * Feat: add support for "hint" property
             * Fix: @angular/cli is not handling "required" and "default" properties
             */
            .forEach(option => {
            var _a;
            const props = (schemaJson === null || schemaJson === void 0 ? void 0 : schemaJson['properties'])[option.name];
            if (props === null || props === void 0 ? void 0 : props['hint']) {
                option.hint = props === null || props === void 0 ? void 0 : props['hint'];
            }
            if (props === null || props === void 0 ? void 0 : props['default']) {
                option.default = props === null || props === void 0 ? void 0 : props['default'];
            }
            if ((_a = schemaJson === null || schemaJson === void 0 ? void 0 : schemaJson['required']) === null || _a === void 0 ? void 0 : _a.includes(option.name)) {
                option.required = true;
            }
        });
        return options;
    }
    return [];
};
exports.getSchematicSchemaOptions = getSchematicSchemaOptions;
/**
 * Returns all the default options of a specific local or external schematic's schema.
 * @async
 * @param {SchematicContext} context The current schematic context.
 * @param {string} [schematicName="ng-add"] Name of the schematic schema (ex: "drag-drop").
 * @param {string} [packageName="current-schematic-collection-name"] Name of the package containing the schematic schema (ex: "@angular/cli").
 * @param {boolean} [external=false] Whether the schematic is local or external.
 * @returns {Promise<JsonObject>} A JsonObject containing all the default options.
 */
const getSchematicSchemaDefaultOptions = async (context, schematicName = 'ng-add', packageName, external = false) => {
    const schemaOptions = await (0, exports.getSchematicSchemaOptions)(context, schematicName, packageName, external);
    return schemaOptions.reduce((defaultOptions, option) => {
        defaultOptions[option.name] = option.default;
        return defaultOptions;
    }, {});
};
exports.getSchematicSchemaDefaultOptions = getSchematicSchemaDefaultOptions;
